function [u] = disloc(nu, disgeom, xy)
%
% An M-File for the forward dislocation problem.
%    Input: Material Constants
%		nu = Poisson's ratio
%           Dislocation Geometry in a Global E-N-Up coordinate system
%		disgeom(1) = len = fault length in strike direction (km)
%		disgeom(2) = wid = fault width in dip direction (km)
%		disgeom(3) = dep = depth  of lower edge of fault (km)
%		disgeom(4) = dip = dip angle (degrees)
%		disgeom(5) = strik =  strike, clockwise from N (degrees)
%		disgeom(6) = delE  = East offset of midpoint from origin (km)
%		disgeom(7) = delN  = North offset of midpoint from origin (km)
%		disgeom(8) = ss  =  strike slip motion (m)	
%		disgeom(9) = ds  =  dip slip motion (m)	
%		disgeom(10) = op  =  opening  motion (m)	
%    	    Vector of receiver coordinates at which displacement are computed
%		xy  = vector [East (km), North (km)]
%    Output: Displacements in the Global coordinate system
%
slip=norm(disgeom(8:9));
rake= atan2(disgeom(9),disgeom(8))/pi*180;
if rake > 180
	rake = rake -360;
end
cendep = disgeom(3) - disgeom(2) * sind(disgeom(4)) * 0.5 ; % depth of fault centroid


delE=disgeom(6); delN=disgeom(7);

% translate xy coordinate to fault centroid
x = xy(:,1) - delE;
y = xy(:,2) - delN;
[E, N] = meshgrid(x , y);

% call okada85 written by Francos
[Ue,Un,Uz]=okada85(E,N, ...
                        cendep,disgeom(5),disgeom(4),  ...
						disgeom(1),disgeom(2),rake,slip,disgeom(10),  ...
                        nu);

u=[diag(Ue), diag(Un), diag(Uz)];

